@@ -1,2 +1,2 @@
-from . import read
+from . import read, utils
 from .model import Model
